diff --git a/src/Fl_Clock.cxx b/src/Fl_Clock.cxx
index ca7237212..8caf155c2 100644
--- a/src/Fl_Clock.cxx
+++ b/src/Fl_Clock.cxx
@@ -212,11 +212,26 @@ static void tick(void *v) {
   time_t sec;
   int usec;
   Fl::system_driver()->gettime(&sec, &usec);
+  fprintf(stderr, "tick: sec =%11ld, usec=%8d", sec, usec);
+  double delta = (1000000 - usec)/1000000.; // time till next second
+  // if current time is just before full second, show that full second
+  // and wait one more second (STR 3516)
+  if (delta < 0.1) {
+    fprintf(stderr," *** Delta = %8.6f (%6d µsec), value =%11d ***",
+	    delta, (1000000 - usec), (int)((Fl_Clock*)v)->value());
+    // delta += 1.0;
+    // sec++;
+  }
   ((Fl_Clock*)v)->value((ulong)sec);
 
-  // schedule timer event slightly later than the next second (+25 ms)
-  // to prevent multiple timer events if triggered too early (STR 3516)
-  Fl::add_timeout((1025000 - usec)/1000000., tick, v);
+  fprintf(stderr, "\n");
+  fflush(stderr);
+
+#if (1) // normal behavior, works well with patched Fl::add_timeout()
+  Fl::add_timeout(delta, tick, v);
+#else // using Fl::repeat_timeout() (almost) simulates "old behavior"
+  Fl::repeat_timeout(delta, tick, v);
+#endif
 }
 
 int Fl_Clock::handle(int event) {
diff --git a/src/drivers/X11/Fl_X11_Screen_Driver.cxx b/src/drivers/X11/Fl_X11_Screen_Driver.cxx
index d71ef7486..a9f0ca2ec 100644
--- a/src/drivers/X11/Fl_X11_Screen_Driver.cxx
+++ b/src/drivers/X11/Fl_X11_Screen_Driver.cxx
@@ -633,10 +633,15 @@ const char *Fl_X11_Screen_Driver::get_system_scheme()
 
 void Fl_X11_Screen_Driver::add_timeout(double time, Fl_Timeout_Handler cb, void *argp) {
   elapse_timeouts();
+  missed_timeout_by = 0;
   repeat_timeout(time, cb, argp);
 }
 
 void Fl_X11_Screen_Driver::repeat_timeout(double time, Fl_Timeout_Handler cb, void *argp) {
+  if (missed_timeout_by < 0.0) {
+    fprintf(stderr, "-- repeat_timeout(), missed_timeout_by = %9.6f\n", missed_timeout_by);
+    fflush(stderr);
+  }
   time += missed_timeout_by; if (time < -.05) time = 0;
   Timeout* t = free_timeout;
   if (t) {
